//+------------------------------------------------------------------+
//|                                   Record Session High Trader.mq4 |
//|                                               Stephen J Reynolds |
//|                                             https://www.mql5.com |

// Record Session High :
// Here i draw a red dot above candles and in middle of MACD when record 
// session highs have counted 8 bullRSH's or more and a blue dot below 
// candles and in middle of MACD when record session highs have counted 
// 8 bearRSH's or more.  

// RSH is a way of telling when a trend is ready for revesal.
// And possibly the trader might either get out or tighten stop levels.
 
// It would best used as an exit method rather than entry. 
// CANDLE MISUSE RULE : Dont expect reversal.

// Ive also added an arrow that appears on MACD when we get a bull or bear
// signal on MACD. To be used alongside RSH. 

// Note : Divergence would work far better than basic MACD direction.
// I will maybe figure out how to code divergence one day.

//+------------------------------------------------------------------+
#property copyright "Stephen J Reynolds"
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_separate_window

#property indicator_buffers 6

//--- plot MovingAverage
#property indicator_label1  "MACD"
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  Black
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1

//--- plot MovingAverage
#property indicator_label2  "SIGNAL"
#property indicator_type2   DRAW_LINE
#property indicator_color2  Red
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

//---- indicator parameters
extern string SECTION1  = "*** MACD Settings ***";
input int FastPeriod = 12;
input int SlowPeriod = 26;
input int SignalPeriod = 9;
input ENUM_APPLIED_PRICE iMacPrice = PRICE_CLOSE;

extern string SECTION2  = "*** Record Session High Count ***";
input int RSH = 8;     // Record Session High Count 
input int resetRSH = 6;  // Record Session High Reset  

// Turning Alarms on or of. See sound file in Program Files > ETX Capital MT4 > Sounds.
extern string SECTION3  = "*** Alarm Settings ***";
input string SoundFile = "alert.wav";  
input bool playsound = false; // Play Sound
input bool messagebox = false; // Message Box
input bool sendmail = false; // Send eMail
input bool sendnotification = false;  // Send Mobile

// Changing Colors Of Arrows
extern string SECTION4  = "*** Arrow Color Settings ***";
input color bullColor = clrRed;  // Bull Arrow
input color bearColor = clrBlue; // Bear Arrow

double iMACD1[];
double iMACDSignal1[];
double bearRSH[];
double bullRSH[];
double bearMACD[]; // bear arrow
double bullMACD[]; // bull arrow

//+------------------------------------------------------------------+
//| Deletes all objects on chart when Indicator is closed or removed from chart.
//+------------------------------------------------------------------+
int deinit()   // When Indicator is taken of the chart we delete objects
  {
   for(int i=ObjectsTotal()-1; i>=0; i--)   // Returns the number of objects of the specified type in the specified chart.
     {
      string label=ObjectName(i);  // Returns the name of the corresponding object by its index in the objects list.     EG label = MACD_DivergenceLine.0# 1448881200
      
      if((label != "Longs") && (label != "Shorts"))   // Cant work becuase only the names of ibjects are understood in the main int On Calculate block
         continue;                                  // So instead i delete all objects. Unless i can figure out how to make it use global object labels
               
      ObjectDelete(label); // Deletes objects as they are cycled through using the for loop      
     }
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,iMACD1);
   SetIndexBuffer(1,iMACDSignal1);

   SetIndexBuffer(2,bearRSH); // Red arrow heading down on chart    wrong its blue
   SetIndexStyle(2,DRAW_ARROW,0,0,bearColor);
   SetIndexEmptyValue(2,EMPTY_VALUE);
   SetIndexArrow(2,174);
   
   SetIndexBuffer(3,bullRSH);    // Blue arrow heading up on chart     wrong its red
   SetIndexStyle(3,DRAW_ARROW,0,0,bullColor);
   SetIndexEmptyValue(3,EMPTY_VALUE);
   SetIndexArrow(3,174);

   SetIndexBuffer(4,bearMACD);  // Red arrow heading down on seperate chart     right colour wrong direction
   SetIndexStyle(4,DRAW_ARROW,0,0,bearColor);
   SetIndexArrow(4,234);    
  
   SetIndexBuffer(5,bullMACD);  // Blue arrow heading up on seperate chart    right colour wrong direction
   SetIndexStyle(5,DRAW_ARROW,0,0,bullColor);
   SetIndexArrow(5,233);  
      
//---
   return(INIT_SUCCEEDED);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {

   int bars = rates_total - 1;
   if(prev_calculated > 0) bars = rates_total - prev_calculated;       
      
   // Code think through :
   // at 00:00  HIGH returns i at 1.08763 - ? all high values going back 1000 bars. 1 bar back from current. It will count 1000 - 1 and remain 
   // at 1 becuase i counts 1000 - 1 at 00:00   it does this all within 00:00 because the 4th parameter in iLowest() counts the number of 
   // bars from the start bar       
	for(int i = bars; i >= 1; i--)
   {
      RefreshRates();
      
      iMACD1[i] = iMACD(_Symbol,_Period,FastPeriod,SlowPeriod,SignalPeriod,iMacPrice,MODE_MAIN,i);  // Histogram      
		iMACDSignal1[i] = iMACD(_Symbol,_Period,FastPeriod,SlowPeriod,SignalPeriod,iMacPrice,MODE_SIGNAL,i); // Main slower red line
   
      // Object Coordinates
      double HIGH = iHigh(_Symbol,_Period,i);
      double LOW = iLow(_Symbol,_Period,i);
      double OPEN = iOpen(_Symbol,_Period,i);
      double CLOSE = iClose(_Symbol,_Period,i);
      datetime TIME = iTime(NULL,0,i);   
              
      string ob_Longs = "Longs"+TimeToStr(TIME);    
      string ob_Shorts = "Shorts"+TimeToStr(TIME);
      static int objectBuyCount = 0;   
      static int objectSellCount = 0;
      
      // Record Session High Count
      static int highBars = TimeMinute(TimeCurrent());
      highBars++;      
      static int lowBars = TimeMinute(TimeCurrent());   
      lowBars++;      
           
      int lowest = iLowest(_Symbol,_Period,MODE_LOW,lowBars,i);   
      double lowestLow = Low[lowest];
      int highest = iHighest(_Symbol,_Period,MODE_HIGH,highBars,i);   
      double highestHigh = High[highest];
      
      static int bullBarCount = 1; // Main objects to use for counting
      static int bearBarCount = 1;   
      static bool enableBullCount = false;   
      static bool enableBearCount = false; 
      static double bullBarClose = Close[1]; 
      static double bearBarClose = Close[1]; 
      static bool lowestCount = true;
      static bool highestCount = true;
   
      static int resetBullCount = 1; // resetCount is when close isnt higher than barClose (the RSH requirement)  
      static int resetBearCount = 1;

      //+------------------------------------------------------------------+
      //| My Code has to do the following : (Applicable to bears)
      // a) Recognise the current bar high is lower than prev highest high. To also reset previous highest high when needed.
      // b) Record close of current bar only if its bull
      // c) Count each new bull bar
      // d) Count each new non bull bar and recognise if more than x have appeared then reset bullbarcount 
      // Vice versa for bears.                                                                |
      //+------------------------------------------------------------------+

      //+------------------------------------------------------------------+
      // Bull Signal  (EG Price has been bearish, now possible reversal)                                                                
      //+------------------------------------------------------------------+
      if(LOW > lowestLow)  // a) Recognise the current bar low is higher than prev lowest low. To also reset previous lowest low when needed.
      {                              
         enableBullCount = true; // Wrong! This is true every time LOW  is lower than lowest so it will trigger record of close
      }                        // Allows use of next code to record close of bull candle
      else                      //  Only if high is higher than lowest low
      {
         enableBullCount = false;  // If low is lower than lowe low then we switch of
         lowestCount = true;    //  the recording of close by saying enableBearCount = false; 
         bullBarCount = 1;     // resest bearBarCount if we recognise a LOW is < lowest low
         lowBars = 1;   
      }     
        
      if(lowestCount == true && enableBullCount == true && CLOSE > OPEN && CLOSE > lowestLow)
      {
         bullBarClose = CLOSE;      // b) Record close of current bar only if its bear      
         lowestCount = false;    // switches of so we cant use this again until a new 
      }                         // lower low is found or resetBearCount > 6                         
   
      if(CLOSE > OPEN && CLOSE > bullBarClose)   // c) Count each new bear bar   
      {
         bullBarClose = CLOSE;    // Records the next bar close. The above statement wont 
         bullBarCount++;        // as its switched of. Also it counts each bearBarClose
      }               
      
      // Counts each bar only if its close isnt lower than previous RSH barcount
      if(CLOSE < bullBarClose) resetBullCount++;  // d) Count each new non bear bar and recognise if more than x have appeared then reset bearbarcount   
         else resetBullCount = 1;                 // For if the following candles dont close highr we count  
         
      // Resets RSH count and highest high object ready for new high search
      if(resetBullCount > resetRSH)   // resets count if theres been too many closes lower than
      {                        // bearCandleClose.  
         bullBarCount = 1;    // resets bearBarCount 
         lowBars = 1;        // resets our count parameter of highest
      }

      //+------------------------------------------------------------------+
      // Bear Signal    (EG Price has been bullish, now possible reversal)   
      // Its all about spotting bear candles and counting them as we go.
      //+------------------------------------------------------------------+
      if(HIGH < highestHigh) 
      {                              
         enableBearCount = true; 
      }                           
      else                   
      {
         enableBearCount = false;  
         highestCount = true;   
         bearBarCount = 1;     
         highBars = 1;
      }        
      if(highestCount == true && enableBearCount == true && CLOSE < OPEN && CLOSE < highestHigh)
      {
         bearBarClose = CLOSE; 
         highestCount = false; 
      }               
      
      if(CLOSE < OPEN && CLOSE < bearBarClose) 
      {
         bearBarClose = CLOSE;  
         bearBarCount++;       
      }              
      
      if(CLOSE > bearBarClose) resetBearCount++; 
         else resetBearCount = 1;            
    
      if(resetBearCount > resetRSH) 
      {                     
         bearBarCount = 1;  
         highBars = 1;   
      }
      
      static int macdBullCount = TimeMinute(TimeCurrent());
      static int macdBearCount = TimeMinute(TimeCurrent());
      double BULLRSH = LOW - (50 * _Point);  // Makes position of arrow end of candle wick +/- 50pts
      double BEARRSH = HIGH + (100 * _Point);

      // Bear condition
      if(bullBarCount >= RSH) bearRSH[i] = 0; // Here we draw objects on the seperate chart. If I want object on main chart i use ObjectCreate here     
  
      // Bull condition  
      if(bearBarCount >= RSH) bullRSH[i] = -0;                   
      
            
      double iMacdHist1 = iMACD(_Symbol,_Period,FastPeriod,SlowPeriod,SignalPeriod,iMacPrice,MODE_MAIN,i);   
      double iMacdHist2 = iMACD(_Symbol,_Period,FastPeriod,SlowPeriod,SignalPeriod,iMacPrice,MODE_MAIN,i+1);           
      bool macdBull = iMacdHist1 > iMacdHist2;
      bool macdBear = iMacdHist1 < iMacdHist2;      
      
      double bullClose = LOW - (300 * _Point);
      double bearClose = HIGH + (300 * _Point); 
           
		int secCount1 = Seconds(); // So i can make signals sound approx every minute
		
		// Bear condition
      if(macdBear)  
      {  
         macdBearCount++;   
         macdBullCount = 0;
      }
   
      if(macdBearCount == 1 && bullBarCount >= RSH) 
      {
         bearMACD[i] = iMacdHist1 + 0.0002;
         
         if(playsound && secCount1==0) PlaySound(SoundFile);
         if(messagebox && secCount1==0) MessageBox("Place order now?", "Confirmation", MB_YESNO);         
         if(sendmail && secCount1==0) SendMail("RSH signal", "RSH Signal Ready on " + OrderSymbol());
         if(sendnotification && secCount1==0) SendNotification("RSH signal on " + OrderSymbol());                    
      }  

      // Bull condition                  
      if(macdBull)
      {  
         macdBullCount++; // Value okay  
         macdBearCount = 0;
      }

      if(macdBullCount == 1 && bearBarCount >= RSH) 
      {
         bullMACD[i] = iMacdHist1 - 0.0002;
         
         if(playsound && secCount1==0) PlaySound(SoundFile);
         if(messagebox && secCount1==0) MessageBox("Place order now?", "Confirmation", MB_YESNO);         
         if(sendmail && secCount1==0) SendMail("RSH signal", "RSH Signal Ready on " + OrderSymbol());
         if(sendnotification && secCount1==0) SendNotification("RSH signal on " + OrderSymbol());    
      }
 
   }
   
//--- return value of prev_calculated for next call
   return(rates_total);
}


